<?xml version="1.0" encoding="utf-8" standalone="yes"?>
<sch:schema xmlns="http://purl.oclc.org/dsdl/schematron"
            xmlns:sch="http://purl.oclc.org/dsdl/schematron"
            xmlns:dsp="urn:aria-raps:etl-dsp-sales:1.0" 
            xmlns:rapstype="urn:ARIA-Chart-Common-Type-1.0"
            queryBinding="xslt"
            schemaVersion="ISO19757-3">
  <sch:title>ARIA-ETL-DSP schematron file. Validates required reliationships within an DSP Sales File.</sch:title>
  <sch:ns prefix="dsp" uri="urn:aria-raps:etl-dsp-sales:1.0" />
  <sch:ns prefix="rapstype" uri="urn:ARIA-UpTake-Type-1.0" />
  <sch:ns prefix="vals" uri="urn:ARIA-UpTake-Validators-1.0"/>    <!-- ISRC, etc. validation-->
  <sch:ns prefix="xsl" uri="http://www.w3.org/1999/XSL/Transform"/>
   
  <sch:pattern id="provider.checks">
    <sch:title>Global validation of common items within the file.</sch:title>

    <sch:rule context="dsp:Provider">
		<!-- Validate that the Provder entries are not intermixed -->
		<!-- Count the number of items that do and don't have Retailer values -->
		<!-- 
              The expression counts the number of Retailers that are not present or empty, and compares that to 0, and also the number
                  that are present, and compares that > 0
              It then does the opposite and ANDs the 2 results together. It should alsways come out (0 and !0) or (!0 and 0). It is set up
                  so that empty will be counted twice if they exists, and will always cause a false condition...which means it should be fixed.
         -->
		<sch:assert test="(count(//*[(name()='TrackSale' or name()='TrackPlay' or name()='BundleSale') and (not(@Retailer) or @Retailer='')]) = 0 and count(//*[(name()='TrackSale' or name()='TrackPlay' or name()='BundleSale') and @Retailer])  > 0) or (count(//*[(name()='TrackSale' or name()='TrackPlay' or name()='BundleSale') and (not(@Retailer) or @Retailer='')]) > 0 and count(//*[(name()='TrackSale' or name()='TrackPlay' or name()='BundleSale') and @Retailer]) = 0)" diagnostics="provider.invalid.AmbiguousRetailerUsage.diag" />
    </sch:rule>
  </sch:pattern>
   
  <sch:pattern id="tracksale.checks">
    <sch:title>Track ISRCs are validated against embedded function</sch:title>

    <sch:rule context="dsp:TrackSale">
      <!-- If supplied ISRC is value -->
      <sch:assert test="not(@ISRC) or vals:ValidateISRC(@ISRC)" diagnostics="tracksale.invalid.ISRC.diag"/>

      <!-- Track is duplicate, sales are different -->
      <sch:assert test="not(preceding-sibling::dsp:TrackSale[@ISRC = current()/@ISRC and @Sales != current()/@Sales and @Artist = current()/@Artist and ((@Retailer = current()/@Retailer) or ( not(@Retailer) and not(current()/@Retailer)))])" diagnostics="tracksale.invalid.ISRCDupe.diag" />
      
      <!-- Track is duplicate, sales are sale! May be a problem. More severe version of the previous assertion -->
      <sch:assert test="not(preceding-sibling::dsp:TrackSale[@ISRC = current()/@ISRC and @Sales = current()/@Sales and @Artist = current()/@Artist and ((@Retailer = current()/@Retailer) or ( not(@Retailer) and not(current()/@Retailer)))])" diagnostics="tracksale.invalid.ISRCDupeAll.diag" />

      <!-- Track is duplicate, sales are different, artist are the same -->
      <sch:assert test="not(preceding-sibling::dsp:TrackSale[@ISRC = current()/@ISRC and @Sales != current()/@Sales and @Artist != current()/@Artist and ((@Retailer = current()/@Retailer) or ( not(@Retailer) and not(current()/@Retailer)))])" diagnostics="tracksale.invalid.ISRCDupeNewArtist.diag" />

      <!-- Track is duplicate, sales are sale!, artists are different, May be a problem. More severe version of the previous assertion -->
      <sch:assert test="not(preceding-sibling::dsp:TrackSale[@ISRC = current()/@ISRC and @Sales = current()/@Sales and @Artist != current()/@Artist and ((@Retailer = current()/@Retailer) or ( not(@Retailer) and not(current()/@Retailer)))])" diagnostics="tracksale.invalid.ISRCDupeAllNewArtist.diag" />

      <!-- More Customers than Sales...how. -->
      <sch:assert test="not(@Customers > @Sales)" diagnostics="tracksale.invalid.MoreCustThanSales.diag" />
      
      <!-- Critical Error: Incorrect Aggregation within the same Region or Location -->
      
    </sch:rule>
  </sch:pattern>

 <sch:pattern id="trackplay.checks">
    <sch:title>Track Play ISRCs are validated against embedded function</sch:title>

    <sch:rule context="dsp:TrackPlays">
		<!-- Seer that the Threshold is unique for this Region/Location -->
		<sch:assert test="not(preceding-sibling::dsp:TrackPlays[@Threshold = current()/@Threshold])" diagnostics="trackplays.invalid.ThresholdDupe.diag"/>
    </sch:rule>
    <sch:rule context="dsp:TrackPlay">
      <!-- If supplied ISRC is value -->
      <sch:assert test="not(@ISRC) or vals:ValidateISRC(@ISRC)" diagnostics="trackplay.invalid.ISRC.diag"/>

      <!-- Track is duplicate, sales are different -->
      <sch:assert test="not(preceding-sibling::dsp:TrackPlay[@ISRC = current()/@ISRC and @TotalPlays != current()/@TotalPlays and @Artist = current()/@Artist and ((@Retailer = current()/@Retailer) or ( not(@Retailer) and not(current()/@Retailer)))])" diagnostics="trackplay.invalid.ISRCDupe.diag" />
      
      <!-- Track is duplicate, sales are sale! May be a problem. More severe version of the previous assertion -->
      <sch:assert test="not(preceding-sibling::dsp:TrackPlay[@ISRC = current()/@ISRC and @TotalPlays = current()/@TotalPlays and @Artist = current()/@Artist and ((@Retailer = current()/@Retailer) or ( not(@Retailer) and not(current()/@Retailer)))])" diagnostics="trackplay.invalid.ISRCDupeAll.diag" />

      <!-- Track is duplicate, sales are different, artitist are the same -->
      <sch:assert test="not(preceding-sibling::dsp:TrackPlay[@ISRC = current()/@ISRC and @TotalPlays != current()/@TotalPlays and @Artist != current()/@Artist and ((@Retailer = current()/@Retailer) or ( not(@Retailer) and not(current()/@Retailer)))])" diagnostics="trackplay.invalid.ISRCDupeNewArtist.diag" />

      <!-- Track is duplicate, sales are sale!, artists are different, May be a problem. More severe version of the previous assertion -->
      <sch:assert test="not(preceding-sibling::dsp:TrackPlay[@ISRC = current()/@ISRC and @TotalPlays = current()/@TotalPlays and @Artist != current()/@Artist and ((@Retailer = current()/@Retailer) or ( not(@Retailer) and not(current()/@Retailer)))])" diagnostics="trackplay.invalid.ISRCDupeAllNewArtist.diag" />

	<!-- Totals Dont Add up correctly -->
      <sch:assert test="@TotalPlays = (@PurchasedPlays + @FreePlays + @SubscriptionPlays + @OtherPlays)" diagnostics="trackplay.invalid.TotalPlaysNotEqualToSum.diag" />
		
    <!-- More Customers than Sales...how. -->
      <sch:assert test="not(@Customers > @TotalPlays)" diagnostics="trackplay.invalid.MoreCustThanPlays.diag" />
      
      <!-- Critical Error: Incorrect Aggregation within the same Region or Location -->
      
    </sch:rule>
  </sch:pattern>

  <sch:pattern id="bundlesale.checks">
    <sch:title>Bundle Validations</sch:title>

    <sch:rule context="dsp:BundleSale">
      <!-- If supplied APN and GRID is value -->
      <sch:assert test="not(@APN) or vals:ValidateBarCode(@APN)" diagnostics="bundlesale.invalid.APN.diag"/>
      <sch:assert test="not(@GRid) or vals:ValidateBarCode(@GRid)" diagnostics="bundlesale.invalid.GRID.diag"/>
      <sch:assert test="not(@CatalogueNumber) or vals:ValidateCatalogueNumber(@CatalogueNumber)" diagnostics="bundlesale.invalid.CatNo.diag"/>

      <!-- Confirm that there is at least an APN, GRID or Catalogue Number -->
      <sch:assert test="@CatalogueNumber or @APN or @GRid" diagnostics="bundlesale.invalid.NoIdentifier.diag"/>

      <!-- BundleSale is duplicate, sales are different -->
      <sch:assert test="not(preceding-sibling::dsp:BundleSale[@APN = current()/@APN and @Sales != current()/@Sales and ((@Retailer = current()/@Retailer) or ( not(@Retailer) and not(current()/@Retailer)))])" diagnostics="bundlesale.invalid.APNDupe.diag" />
      <!-- Track is duplicate, sales are sale! May be a problem. More severe version of the previous assertion -->
      <sch:assert test="not(preceding-sibling::dsp:BundleSale[@APN = current()/@APN and @Sales = current()/@Sales and ((@Retailer = current()/@Retailer) or ( not(@Retailer) and not(current()/@Retailer)))])" diagnostics="bundlesale.invalid.APNDupeAll.diag" />

      <!-- BundleSale is duplicate, sales are different -->
      <sch:assert test="not(preceding-sibling::dsp:BundleSale[@GRid = current()/@GRid and @Sales != current()/@Sales and ((@Retailer = current()/@Retailer) or ( not(@Retailer) and not(current()/@Retailer)))])" diagnostics="bundlesale.invalid.GRIDDupe.diag" />
      <!-- Track is duplicate, sales are sale! May be a problem. More severe version of the previous assertion -->
      <sch:assert test="not(preceding-sibling::dsp:BundleSale[@GRid = current()/@GRid and @Sales = current()/@Sales and ((@Retailer = current()/@Retailer) or ( not(@Retailer) and not(current()/@Retailer)))])" diagnostics="bundlesale.invalid.GRIDDupeAll.diag" />

      <!-- BundleSale is duplicate, sales are different -->
      <sch:assert test="not(preceding-sibling::dsp:BundleSale[@CatalogueNumber = current()/@CatalogueNumber and @Sales != current()/@Sales and ((@Retailer = current()/@Retailer) or ( not(@Retailer) and not(current()/@Retailer)))])" diagnostics="bundlesale.invalid.CatNoDupe.diag" />
      <!-- Track is duplicate, sales are sale! May be a problem. More severe version of the previous assertion -->
      <sch:assert test="not(preceding-sibling::dsp:BundleSale[@CatalogueNumber = current()/@CatalogueNumber and @Sales = current()/@Sales and ((@Retailer = current()/@Retailer) or ( not(@Retailer) and not(current()/@Retailer)))])" diagnostics="bundlesale.invalid.CatNoDupeAll.diag" />

      <!-- More Customers than Sales...how. -->
      <sch:assert test="not(@Customers > @Sales)" diagnostics="bundlesale.invalid.MoreCustThanSales.diag" />
      
      <!-- Critical Error: Incorrect Aggregation within the same Region or Location -->
      
    </sch:rule>
  </sch:pattern>

  <sch:diagnostics>
    <!-- Global Assertions about the file being correct or otherwise. -->  
    <!-- Mixed Retailer values -->
    <sch:diagnostic id="provider.invalid.AmbiguousRetailerUsage.diag">The file contains mixed usage of the Retailer attribute. The Retailer is optional, however, it may only be ommited when there is 1 retailer. If multiple reatailers are reported within the single file, then all sales must have a qualifying Retailer set.</sch:diagnostic>
    
    <!-- Track Sale Validation Errors -->
    <sch:diagnostic id="tracksale.invalid.ISRC.diag">The referenced ISRC supplied within the file was not valid. TrackSale[ParentElement/@Name=<xsl:value-of select="ancestor::dsp:Region/@Name" /><xsl:value-of select="ancestor::dsp:Location/@Name" />, @ISRC=<xsl:value-of select="@ISRC" />, @Sales=<xsl:value-of select="@Sales" />]</sch:diagnostic>
    <sch:diagnostic id="tracksale.invalid.ISRCDupe.diag"><xsl:if test="not(@Retailer)">The referenced ISRC is duplicate within the same Region or Location. TrackSale[ParentElement/@Name=<xsl:value-of select="ancestor::dsp:Region/@Name" /><xsl:value-of select="ancestor::dsp:Location/@Name" />, @ISRC=<xsl:value-of select="@ISRC" />, @Sales=<xsl:value-of select="@Sales" />]</xsl:if><xsl:if test="@Retailer">The referenced ISRC is duplicate within the same Region or Location, using the same @Retailer value. TrackSale[ParentElement/@Name=<xsl:value-of select="ancestor::dsp:Region/@Name" /><xsl:value-of select="ancestor::dsp:Location/@Name" />, @ISRC=<xsl:value-of select="@ISRC" />, @Sales=<xsl:value-of select="@Sales" />, @Retailer=<xsl:value-of select="@Retailer" />]</xsl:if></sch:diagnostic>
    <sch:diagnostic id="tracksale.invalid.ISRCDupeAll.diag"><xsl:if test="not(@Retailer)">The referenced ISRC is duplicate and has the same sales within the same Region or Location. TrackSale[ParentElement/@Name=<xsl:value-of select="ancestor::dsp:Region/@Name" /><xsl:value-of select="ancestor::dsp:Location/@Name" />, @ISRC=<xsl:value-of select="@ISRC" />, @Sales=<xsl:value-of select="@Sales" />]</xsl:if><xsl:if test="@Retailer">The referenced ISRC is duplicate and has the same sales within the same Region or Location, using the same @Retailer value. . TrackSale[ParentElement/@Name=<xsl:value-of select="ancestor::dsp:Region/@Name" /><xsl:value-of select="ancestor::dsp:Location/@Name" />, @ISRC=<xsl:value-of select="@ISRC" />, @Sales=<xsl:value-of select="@Sales" />, @Retailer=<xsl:value-of select="@Retailer" />]</xsl:if></sch:diagnostic>

    <sch:diagnostic id="tracksale.invalid.MoreCustThanSales.diag">The TrackSale has more customers than sales. How?. TrackSale[ParentElement/@Name=<sch:value-of select="ancestor::dsp:Region/@Name" /><sch:value-of select="ancestor::dsp:Location/@Name" />, @ISRC=<sch:value-of select="@ISRC" />, @Customers=<sch:value-of select="@Customers"/>, @Sales=<sch:value-of select="@Sales" />]</sch:diagnostic>

    <sch:diagnostic id="tracksale.invalid.ISRCDupeNewArtist.diag">The referenced ISRC is duplicate within the same Region or Location, with different Artist. TrackSale[ParentElement/@Name=<sch:value-of select="ancestor::dsp:Region/@Name" /><sch:value-of select="ancestor::dsp:Location/@Name" />, @ISRC=<sch:value-of select="@ISRC" />, @Sales=<sch:value-of select="@Sales" />]</sch:diagnostic>
    <sch:diagnostic id="tracksale.invalid.ISRCDupeAllNewArtist.diag">The referenced ISRC is duplicate and has the same sales within the same Region or Location, with different Artists. TrackSale[ParentElement/@Name=<sch:value-of select="ancestor::dsp:Region/@Name" /><sch:value-of select="ancestor::dsp:Location/@Name" />, @ISRC=<sch:value-of select="@ISRC" />, @Sales=<sch:value-of select="@Sales" />]</sch:diagnostic>

    <!-- TODO: Fixed clarification wrong Retailer combinations -->
    <!--sch:diagnostic id="tracksale.invalid.ISRCAmbiguousOptionalRetailer.diag">The referenced ISRC is duplicate within the same Region or Location. TrackSale[ParentElement/@Name=<sch:value-of select="ancestor::dsp:Region/@Name" /><sch:value-of select="ancestor::dsp:Location/@Name" />, @ISRC=<sch:value-of select="@ISRC" />, @Sales=<sch:value-of select="@Sales" />]</sch:diagnostic-->


    <!-- Track Plays Validation errors -->
    <sch:diagnostic id="trackplays.invalid.ThresholdDupe.diag">Duplicate TrackPlays exist within the same file, with the same Threashold value. The grouping of TrackPlays is incorrect.TrackPlayd[ParentElement/@Name=<sch:value-of select="ancestor::dsp:Region/@Name" /><sch:value-of select="ancestor::dsp:Location/@Name" />, @Threshold=<sch:value-of select="@Threshold" />]</sch:diagnostic>

    <!-- Track Play Validation Errors -->
    <sch:diagnostic id="trackplay.invalid.ISRC.diag">The referenced ISRC supplied within the file was not valid. TrackPlay[ParentElement/@Name=<sch:value-of select="ancestor::dsp:Region/@Name" /><sch:value-of select="ancestor::dsp:Location/@Name" />, @ISRC=<sch:value-of select="@ISRC" />, @TotalPlays=<sch:value-of select="@TotalPlays" />]</sch:diagnostic>
    <sch:diagnostic id="trackplay.invalid.ISRCDupe.diag"><xsl:if test="not(@Retailer)">The referenced ISRC is duplicate within the same Region, Location or TrackPlays. TrackPlay[ParentElement/@Name=<xsl:value-of select="ancestor::dsp:Region/@Name" /><xsl:value-of select="ancestor::dsp:Location/@Name" />, @ISRC=<xsl:value-of select="@ISRC" />, @TotalPlays=<xsl:value-of select="@TotalPlays" />]</xsl:if><xsl:if test="@Retailer">The referenced ISRC is duplicate within the same Region, Location or TrackPlays, using the same @Retailer value. TrackPlay[ParentElement/@Name=<xsl:value-of select="ancestor::dsp:Region/@Name" /><xsl:value-of select="ancestor::dsp:Location/@Name" />, @ISRC=<xsl:value-of select="@ISRC" />, @TotalPlays=<xsl:value-of select="@TotalPlays" />, @Retailer=<xsl:value-of select="@Retailer" />]</xsl:if></sch:diagnostic>
    <sch:diagnostic id="trackplay.invalid.ISRCDupeAll.diag"><xsl:if test="not(@Retailer)">The referenced ISRC is duplicate and has the same sales within the same Region, Location or TrackPlays. TrackPlay[ParentElement/@Name=<xsl:value-of select="ancestor::dsp:Region/@Name" /><xsl:value-of select="ancestor::dsp:Location/@Name" />, @ISRC=<xsl:value-of select="@ISRC" />, @TotalPlays=<xsl:value-of select="@TotalPlays" />]</xsl:if><xsl:if test="@Retailer">The referenced ISRC is duplicate and has the same sales within the same Region, Location or TrackPlays, using the same @Retailer value. TrackPlay[ParentElement/@Name=<xsl:value-of select="ancestor::dsp:Region/@Name" /><xsl:value-of select="ancestor::dsp:Location/@Name" />, @ISRC=<xsl:value-of select="@ISRC" />, @TotalPlays=<xsl:value-of select="@TotalPlays" />, @Retailer=<xsl:value-of select="@Retailer" />]</xsl:if></sch:diagnostic>
    <sch:diagnostic id="trackplay.invalid.MoreCustThanPlays.diag">The TrackSale has more customers than sales. How?. TrackPlay[ParentElement/@Name=<sch:value-of select="ancestor::dsp:Region/@Name" /><sch:value-of select="ancestor::dsp:Location/@Name" />, @ISRC=<sch:value-of select="@ISRC" />, @Customers=<sch:value-of select="@Customers"/> @TotalPlays=<sch:value-of select="@TotalPlays" />]</sch:diagnostic>

	<!-- Numerical checks on values -->
    <sch:diagnostic id="trackplay.invalid.TotalPlaysNotEqualToSum.diag">The TotalPlays quantity does not sum to the other PlayQuantities. TrackPlay[ParentElement/@Name=<sch:value-of select="ancestor::dsp:Region/@Name" /><sch:value-of select="ancestor::dsp:Location/@Name" />, @ISRC=<sch:value-of select="@ISRC" />, @TotalPlays=<sch:value-of select="@TotalPlays" />]</sch:diagnostic>

    <sch:diagnostic id="trackplay.invalid.ISRCDupeNewArtist.diag">The referenced ISRC is duplicate within the same Region or Location, with different Artist. TrackPlay[ParentElement/@Name=<sch:value-of select="ancestor::dsp:Region/@Name" /><sch:value-of select="ancestor::dsp:Location/@Name" />, @ISRC=<sch:value-of select="@ISRC" />, @TotalPlays=<sch:value-of select="@TotalPlays" />]</sch:diagnostic>
    <sch:diagnostic id="trackplay.invalid.ISRCDupeAllNewArtist.diag">The referenced ISRC is duplicate and has the same sales within the same Region or Location, with different Artists. TrackPlay[ParentElement/@Name=<sch:value-of select="ancestor::dsp:Region/@Name" /><sch:value-of select="ancestor::dsp:Location/@Name" />, @ISRC=<sch:value-of select="@ISRC" />, @TotalPlays=<sch:value-of select="@TotalPlays" />]</sch:diagnostic>



    <!-- Duplicate Product Identifier Messages. Tests Main and Alt Identifiers -->
    <sch:diagnostic id="bundlesale.invalid.GRIDDupe.diag"><xsl:if test="not(@Retailer)">The referenced GRID is duplicate within the same Region or Location. BundleSale[ParentElement/@Name=<xsl:value-of select="ancestor::dsp:Region/@Name" /><xsl:value-of select="ancestor::dsp:Location/@Name" />, @GRid=<xsl:value-of select="@GRid" />]</xsl:if><xsl:if test="@Retailer">The referenced GRID is duplicate within the same Region or Location, using the same @Retailer value. BundleSale[ParentElement/@Name=<xsl:value-of select="ancestor::dsp:Region/@Name" /><xsl:value-of select="ancestor::dsp:Location/@Name" />, @GRid=<xsl:value-of select="@GRid" />, @Retailer=<xsl:value-of select="@Retailer" />]</xsl:if></sch:diagnostic>
    <sch:diagnostic id="bundlesale.invalid.APNDupe.diag"><xsl:if test="not(@Retailer)">The referenced APN is duplicate within the same Region or Location. BundleSale[ParentElement/@Name=<xsl:value-of select="ancestor::dsp:Region/@Name" /><xsl:value-of select="ancestor::dsp:Location/@Name" />, @APN=<xsl:value-of select="@APN" />]</xsl:if><xsl:if test="@Retailer">The referenced APN is duplicate within the same Region or Location, using the same @Retailer value. BundleSale[ParentElement/@Name=<xsl:value-of select="ancestor::dsp:Region/@Name" /><xsl:value-of select="ancestor::dsp:Location/@Name" />, @APN=<xsl:value-of select="@APN" />, @Retailer=<xsl:value-of select="@Retailer" />]</xsl:if></sch:diagnostic>
    <sch:diagnostic id="bundlesale.invalid.CatNoDupe.diag"><xsl:if test="not(@Retailer)">The referenced Catalogue Number is duplicate within the same Region or Location. BundleSale[ParentElement/@Name=<xsl:value-of select="ancestor::dsp:Region/@Name" /><xsl:value-of select="ancestor::dsp:Location/@Name" />, @CatalogueNumber=<xsl:value-of select="@CatalogueNumber" />]</xsl:if><xsl:if test="@Retailer">The referenced Catalogue Number is duplicate within the same Region or Location, using the same @Retailer value. BundleSale[ParentElement/@Name=<xsl:value-of select="ancestor::dsp:Region/@Name" /><xsl:value-of select="ancestor::dsp:Location/@Name" />, @CatalogueNumber=<xsl:value-of select="@CatalogueNumber" />, @Retailer=<xsl:value-of select="@Retailer" />]</xsl:if></sch:diagnostic>

    <!-- Duplicate Product Identifier Messages. Tests Main and Alt Identifiers -->
    <sch:diagnostic id="bundlesale.invalid.GRIDDupeAll.diag"><xsl:if test="not(@Retailer)">The referenced GRID is duplicate and has the same sales within the same Region or Location. BundleSale[ParentElement/@Name=<xsl:value-of select="ancestor::dsp:Region/@Name" /><xsl:value-of select="ancestor::dsp:Location/@Name" />, @GRid=<xsl:value-of select="@GRid" />, @Sales=<xsl:value-of select="@Sales" />]</xsl:if><xsl:if test="@Retailer">The referenced GRID is duplicate and has the same sales within the same Region or Location, using the same @Retailer value. BundleSale[ParentElement/@Name=<xsl:value-of select="ancestor::dsp:Region/@Name" /><xsl:value-of select="ancestor::dsp:Location/@Name" />, @GRid=<xsl:value-of select="@GRid" />, @Sales=<xsl:value-of select="@Sales" />, @Retailer=<xsl:value-of select="@Retailer" />]</xsl:if></sch:diagnostic>
    <sch:diagnostic id="bundlesale.invalid.APNDupeAll.diag"><xsl:if test="not(@Retailer)">The referenced APN is duplicate and has the same sales within the same Region or Location. BundleSale[ParentElement/@Name=<xsl:value-of select="ancestor::dsp:Region/@Name" /><xsl:value-of select="ancestor::dsp:Location/@Name" />, @APN=<xsl:value-of select="@APN" />, @Sales=<xsl:value-of select="@Sales" />]</xsl:if><xsl:if test="@Retailer">The referenced APN is duplicate and has the same sales within the same Region or Location, using the same @Retailer value. BundleSale[ParentElement/@Name=<xsl:value-of select="ancestor::dsp:Region/@Name" /><xsl:value-of select="ancestor::dsp:Location/@Name" />, @APN=<xsl:value-of select="@APN" />, @Sales=<xsl:value-of select="@Sales" />, @Retailer=<xsl:value-of select="@Retailer" />]</xsl:if></sch:diagnostic>
    <sch:diagnostic id="bundlesale.invalid.CatNoDupeAll.diag"><xsl:if test="not(@Retailer)">The referenced Catalogue Number is duplicate and has the same sales within the same Region or Location. BundleSale[ParentElement/@Name=<xsl:value-of select="ancestor::dsp:Region/@Name" /><xsl:value-of select="ancestor::dsp:Location/@Name" />, @CatalogueNumber=<xsl:value-of select="@CatalogueNumber" />, @Sales=<xsl:value-of select="@Sales" />]</xsl:if><xsl:if test="@Retailer">The referenced Catalogue Number is duplicate and has the same sales within the same Region or Location, using the same @Retailer value. BundleSale[ParentElement/@Name=<xsl:value-of select="ancestor::dsp:Region/@Name" /><xsl:value-of select="ancestor::dsp:Location/@Name" />, @CatalogueNumber=<xsl:value-of select="@CatalogueNumber" />, @Sales=<xsl:value-of select="@Sales" />, @Retailer=<xsl:value-of select="@Retailer" />]</xsl:if></sch:diagnostic>

    <!-- Number validation algorithm errors -->
    <sch:diagnostic id="bundlesale.invalid.GRID.diag">The referenced GRID supplied within the file was not valid. BundleSale[ParentElement/@Name=<xsl:value-of select="ancestor::dsp:Region/@Name" /><xsl:value-of select="ancestor::dsp:Location/@Name" />, @GRid=<xsl:value-of select="@GRid" />]</sch:diagnostic>
    <sch:diagnostic id="bundlesale.invalid.APN.diag">The referenced APN supplied within the file was not valid. BundleSale[ParentElement/@Name=<xsl:value-of select="ancestor::dsp:Region/@Name" /><xsl:value-of select="ancestor::dsp:Location/@Name" />, @APN=<xsl:value-of select="@APN" />]</sch:diagnostic>
    <sch:diagnostic id="bundlesale.invalid.CatNo.diag">The referenced Catalogue Number supplied within the file was not valid. Possibly missing or empty. BundleSale[ParentElement/@Name=<xsl:value-of select="ancestor::dsp:Region/@Name" /><xsl:value-of select="ancestor::dsp:Location/@Name" />, @ProductCatalogueNumber=<xsl:value-of select="@ProductCatalogueNumber" />]</sch:diagnostic>

    <!-- No Identifiers -->
    <sch:diagnostic id="bundlesale.invalid.NoIdentifier.diag">The BundleSale has NO Sales Identifier. BundleSale[ParentElemenet/@Name=<sch:value-of select="ancestor::dsp:Region/@Name" /><sch:value-of select="ancestor::dsp:Location/@Name" />, @Title=<sch:value-of select="@Title" />, @Artist=<sch:value-of select="@Artist" />]</sch:diagnostic>

    <!-- Customers > Sales -->
    <sch:diagnostic id="bundlesale.invalid.MoreCustThanSales.diag">The BundleSale has more customers than sales. How?. BundleSale[ParentElement/@Name=<sch:value-of select="ancestor::dsp:Region/@Name" /><sch:value-of select="ancestor::dsp:Location/@Name" />, @CatalogueNumber=<sch:value-of select="@CatalogueNumber" />, @APN=<sch:value-of select="@APN" />, @GRid=<sch:value-of select="@GRid" />, @Customers=<sch:value-of select="@Customers"/>, @Sales=<sch:value-of select="@Sales" />]</sch:diagnostic>
  </sch:diagnostics>
</sch:schema>